@@ -54,14 +54,6 @@
 /* Define to rpl_mktime if the replacement function should be used.  */
 #undef mktime
 
-/* Define to rpl_getopt if the replacement function should be used,
- * And define the rpl_* if the replacement getopt is used */
-#undef getopt
-#undef optarg
-#undef opterr
-#undef optind
-#undef optopt
-
 /* Define to rpl_exit if the replacement function should be used.  */
 #undef exit
 
