@@ -267,7 +267,6 @@ a2_read_config (a2ps_job * job, const char *path, const char *file)
       else if (strequ (token, "FileCommand:"))
 	{
 	  token2 = GET_LINE_TOKEN (NULL);
-	  CHECK_TOKEN ();
 	  xstrcpy (job->file_command, token2);
 	}
       else if (strequ (token, "Pattern:")
@@ -295,44 +294,34 @@ a2_read_config (a2ps_job * job, const char *path, const char *file)
  *    to check the package before the installation.  The worst
  *    case is when an older a2ps is yet installed.  So we _must_
  *    have a special way to deal with the tests.  This is why
- *    I introduced yet another env-var: NO_SYSCONF, which
- *    should _not_ be documented, but used with `make check'
+ *    I introduced an env-var: A2PS_CONFIG, which
+ *    points to a2ps.cfg
  *    Note that it also improves the robustness of `make distcheck'
  */
 void
 a2_read_sys_config (a2ps_job * job)
 {
   char * config_file;
-  char ** tmp_path;
-  int saved_errno;
 
-  if (! (getenv ("NO_SYSCONF")))
-    /* Well, we are not being tested, so the config file should
-     * be in SYSCONFDIR.  I see no reason to end a2ps here
-     * if the file is not found: other files follow.  Just say it.
-     */
-    if (a2_read_config (job, SYSCONFDIR, "a2ps.cfg"))
-      return;
-  
-  /* We cannot find sys' config file.  Remember why in case we
-   * cannot find it elsewhere */
-  saved_errno = errno;
-
-  /* Maybe the sys' config file is in the path A2PS_LIBRARY */
-  tmp_path = pw_string_to_path (getenv ("A2PS_LIBRARY"), PATH_SEP);
-  config_file = pw_find_file (tmp_path, "a2ps.cfg", NULL);
-  
-  if (!config_file)
-    /* It is not.  Report an error about the sys' conf, and
-     * the one in the path */
-    error (1, saved_errno,
-	   _("cannot find configuration file `%s'"), "a2ps.cfg");
-  
-  /* We found it in the path.  Read it */
-  if (!a2_read_config (job, NULL, config_file))
-    error (0, errno,
-	   _("cannot open configuration file `%s'"), config_file);
-
-  pw_free_path (tmp_path);
-  XFREE (config_file);
+  if ((config_file = getenv ("A2PS_CONFIG")))
+    {
+      if (a2_read_config (job, NULL, config_file))
+	return;
+      error (0, errno,
+	     _("cannot open configuration file `%s'"), config_file);
+    }
+  else
+    {
+      /* Well, we are not being tested, so the config file should
+       * be in SYSCONFDIR.  I see no reason to end a2ps here
+       * if the file is not found: other files follow.  Just say it.
+       */
+      if (a2_read_config (job, SYSCONFDIR, "a2ps.cfg"))
+	return;
+      config_file = ALLOCA (char, 
+			    strlen (SYSCONFDIR) + strlen ("a2ps.cfg") + 2);
+      sprintf (config_file, "%s/%s", SYSCONFDIR, "a2ps.cfg");
+      error (0, errno,
+	     _("cannot open configuration file `%s'"), config_file);
+    }
 }
