@@ -998,7 +998,7 @@ unsigned int
 char_WX (a2ps_job * job, uchar c)
 {
 #define _WX_(char)  (job->encoding->faces_wx[job->status->face][(int) char])
-  int wx = _WX_(c);
+  unsigned int wx = _WX_(c);
 
   /* Only printable characters have a positive wx */
   if (wx)
@@ -1027,17 +1027,37 @@ char_WX (a2ps_job * job, uchar c)
     return _WX_('?');
   
   case caret:
-    return(0177 < c ? (_WX_('M') + _WX_('-')): 0 
-	   + (((c&0177) < ' ' || (c&0177) == 0177) ? _WX_('^') : 0)
-	   + wx);
+    if (0177 < c)
+      {
+	wx += _WX_('M') + _WX_('-');
+	c &= 0177;
+      }
+    if (c == 0177)
+      wx += _WX_('^') + _WX_('?');
+    else if (c < ' ')
+      wx += _WX_ ('^') + _WX_ (c + '@');
+    else
+      wx += _WX_ (c);
+    return wx;
 
   case Emacs:
-    return(0177 < c ? (_WX_('M') + _WX_('-')): 0 
-	   + (((c&0177) < ' ' || (c&0177) == 0177) ? (_WX_('C') + _WX_ ('-')): 0)
-	   + wx);
+    if (0177 < c)
+      {
+	wx += _WX_('M') + _WX_('-');
+	c &= 0177;
+      }
+    if (c == 0177)
+      wx += _WX_('C') + _WX_('-') + _WX_('?');
+    else if (c < ' ')
+      wx += _WX_('C') + _WX_('-') + _WX_ (c + '@');
+    else
+      wx += _WX_ (c);
+    return wx;
 
+  default:
+    abort ();
   }
-  error (1, 0, "char_WX");
+
   return 0;	/* For -Wall */
 }
 
