@@ -92,8 +92,8 @@ hash_find_slot (struct hash_table* ht, void const *key)
       hash_1 %= ht->ht_size;
       slot = &ht->ht_vec[hash_1];
 
-      if (*slot == 0)
-	return slot;
+      if (*slot == 0) 
+	return deleted_slot ? deleted_slot : slot;
       if (*slot == hash_deleted_item)
 	{
 	  if (deleted_slot == 0)
