@@ -190,7 +190,9 @@ a2ps_job_new (void)
 
   /* Set the NLS on */
   setlocale (LC_TIME, "");
+#ifdef HAVE_LC_MESSAGES
   setlocale (LC_MESSAGES, "");
+#endif
   setlocale (LC_COLLATE, "");
   
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -213,11 +215,10 @@ a2ps_job_new (void)
   /* Short cuts defined by the user */
   res->user_options = user_options_table_new ();
 
-  /* These are declared as global in backupfile.c	*/
+  /* Honor what the user said in its environment */
   if ((cp = getenv ("SIMPLE_BACKUP_SUFFIX")))
     simple_backup_suffix = cp;
-  backup_type = a2ps_get_backup_type (getenv ("VERSION_CONTROL"));
-	  		/* Honor what the user said in its environment	*/
+  res->backup_type = a2ps_get_backup_type (getenv ("VERSION_CONTROL"));
   
   /* Get current time information */
   tim = time (NULL);
@@ -231,7 +232,7 @@ a2ps_job_new (void)
   res->duplex = false;
   res->columns = 1;
   res->rows = 1;
-  res->Major = major_rows;	/* by default, write horizontally	*/
+  res->madir = madir_rows;	/* by default, write horizontally	*/
   res->virtual = 0;
   res->copies = 1;
   res->margin = 0;
