@@ -42,11 +42,8 @@
 #include "caret.h"
 #include "common.h"
 #include "printers.h"
-
-/* Should the virtual page go first to the left, or the bottom? */
-typedef enum {
-  major_rows, major_columns
-} MAJOR;
+#include "backupfile.h"
+#include "madir.h"
 
 struct stream;
 
@@ -77,6 +74,7 @@ typedef struct a2ps_job {
 
   /* Relative to the whole process */
   struct tm run_tm; 		/* Time when this program is run	*/
+  enum backup_type backup_type;	/* When to backup output files		*/
   int sheets;			/* number of sheets before the curr file*/
   int pages;			/* id. but pages			*/
   int total_files;		/* id but files				*/
@@ -84,7 +82,7 @@ typedef struct a2ps_job {
   boolean duplex;		/* Recto verso printing ? 		*/
   int columns;			/* number of columns/page		*/
   int rows;			/* number of rows/page			*/
-  MAJOR Major;			/* firt rows, or columns?
+  madir_t madir;		/* firt rows, or columns?
 				 * (not major, because of a SunOS macro)*/
   int virtual;			/* number of the virtual page on that sheet */
   int copies;			/* number of copies			*/
