@@ -407,7 +407,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      {
 		/* `%D' run date in `yy-mm-dd' format */
 		sprintf ((char *)buf, "%02d-%02d-%02d", 
-			 job->run_tm.tm_year,
+			 job->run_tm.tm_year % 100,
 			 job->run_tm.tm_mon + 1, 
 			 job->run_tm.tm_mday);
 	      }
@@ -546,7 +546,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    sprintf ((char *)buf, "%02d/%02d/%02d", 
 		     job->run_tm.tm_mon + 1,
 		     job->run_tm.tm_mday, 
-		     job->run_tm.tm_year);
+		     job->run_tm.tm_year % 100);
 	    APPEND_STR (buf);
 	    break;
 
@@ -723,7 +723,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	      {
 		/* `$D' mod date in `yy-mm-dd' format */
 		sprintf ((char *)buf, "%02d-%02d-%02d",
-			 file->mod_tm.tm_year,
+			 file->mod_tm.tm_year % 100,
 			 file->mod_tm.tm_mon + 1,
 			 file->mod_tm.tm_mday);
 	      }
@@ -977,7 +977,7 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 	    sprintf ((char *)buf, "%02d/%02d/%02d", 
 		     file->mod_tm.tm_mon + 1,
 		     file->mod_tm.tm_mday, 
-		     file->mod_tm.tm_year);
+		     file->mod_tm.tm_year % 100);
 	    APPEND_STR (buf);
 	    break;
 
@@ -1146,8 +1146,8 @@ grow_user_string_obstack (struct obstack * user_string_stack,
 		test = message_verbosity == 0;
 		break;
 
-	      case 'r': /* `#?r' Major = row		*/
-		test = job->Major == major_rows;
+	      case 'r': /* `#?r' madir = row		*/
+		test = job->madir == madir_rows;
 		break;
 
 	      case 'V':	/* `#?V' verbose mode */
