@@ -47,6 +47,8 @@
 #include "useropt.h"
 #include "prange.h"
 #include "metaseq.h"
+#include "madir.h"
+#include "getnum.h"
 
 #define MAN_LINES               66	/* no lines for a man */
 extern char * program_name;
@@ -58,16 +60,6 @@ option_hook handle_option_hook = NULL;
 
 
 
-static void
-print_argv (const char * string, char * argv[])
-{
-  int i;
-  fprintf (stderr, "%2d %-10s ", optind, string);
-  for (i = 0 ; argv [i] ; i++)
-    fprintf (stderr, "%-10s", argv[i]);
-  fputs ("\n", stderr);
-}
-
 static struct option long_options[] =
 {
   /* Tasks */
@@ -85,6 +77,8 @@ static struct option long_options[] =
 
   /* Global */
   {"macro-meta-sequence",required_argument,	0, 137},
+  {"variable",		required_argument,	0, 137},
+  {"define",		required_argument,	0, 137},
   {"user-option",	required_argument,	0, '='},
 
   /* Pretty Print */
@@ -175,54 +169,6 @@ static struct option long_options[] =
 /************************************************************************/
 /*				arguments				*/
 /************************************************************************/
-/*
- * Return the index of the right answer
- */
-int
-a2ps_get_symbolic_value (const char * const args[], 
-			 const void * values,
-			 const char * opt,
-			 const char * arg)
-{
-  int i, last_value = 0;
-  
-  i = argmatch (arg, args);
-
-  if (i >= 0)
-    return i;
-
-  /* There is an error */
-  if (i == -1)
-    error (0, 0, _("invalid argument `%s' for option `%s'"),
-	   arg, opt);
-  else				/* Assume -2.  */
-    error (0, 0, _("ambiguous argument `%s' for option `%s'"),
-	   arg, opt);
-
-  /* We try to put synonyms on the same line.
-   * The assumption is that they follow each other */
-  fprintf (stderr, _("Valid arguments are:"));
-  for (i = 0 ; args[i] ; i++)
-    if ((i == 0)
-	|| (last_value != ((const int *) values)[i])) {
-      fprintf (stderr, "\n  * `%s'", args[i]);
-      last_value = ((const int *) values)[i];
-    } else {
-      fprintf (stderr, ", `%s'", args[i]);
-    }
-  putc ('\n', stderr);
-  exit (EXIT_BADARG);
-  return -1; 	/* To please some compilers */
-}
-
-static void
-bad_arg (const char * opt, char * arg)
-{
-  error (0, 0, _("wrong value for option %s: `%s'"), opt, arg);
-  fprintf (stderr, _("Try `%s --help' for more information.\n"), program_name);
-  exit (EXIT_BADARG);
-}
-
 /************************************************************************/
 /* Helping routines for the options' arguments				*/
 /************************************************************************/
@@ -252,13 +198,11 @@ static const enum backup_type backup_types[] =
 enum backup_type
 a2ps_get_backup_type (const char *version)
 {
-  int i;
-  
   if (version == 0 || *version == 0)
     return numbered_existing;
-  i = a2ps_get_symbolic_value (backup_args, backup_types, 
-			       "--version-control", version);
-  return backup_types[i];
+  else
+    return XARGCASEMATCH ("--version-control", version, 
+                          backup_args, backup_types);
 }
 
 /*
@@ -283,37 +227,9 @@ static const boolean boolean_types[] =
 boolean
 a2ps_get_boolean (const char * option, const char * arg)
 {
-  int i = a2ps_get_symbolic_value (boolean_args, boolean_types,
-			      option, arg);
-  return boolean_types[i];
+  return XARGCASEMATCH (option, arg, boolean_args, boolean_types);
 }
 
-int
-a2ps_get_integer_in_range (const char * option, const char * arg,
-			   int min, int max)
-{
-  char buf[256];
-  int res;
-
-  if ((sscanf (optarg, "%d%255s", &res, buf) != 1)
-      || ((min >= 0) && (res < min))
-      || ((max >= 0) && (res > max)))
-    {
-      error (0, 0, _("invalid argument `%s' for option `%s'"),
-	     arg, option);
-      if (min >= 0 && max >= 0)
-	fprintf (stderr, _("Valid arguments are integers between %d and %d\n"),
-		 min, max);
-      else if (min >= 0)
-	fprintf (stderr, _("Valid arguments are integers greater than %d\n"),
-		 min);
-      else
-	fprintf (stderr, _("Valid arguments are integers smaller than %d\n"),
-		 max);
-      exit (EXIT_BADARG);
-    }
-  return res;
-}
 
 /*
  * Return the non printable format
@@ -342,87 +258,7 @@ static const enum unprintable_format non_printable_types[] =
 static enum unprintable_format
 get_non_printable_format (const char * option, const char * arg)
 {
-  int i = a2ps_get_symbolic_value (non_printable_args, non_printable_types,
-				  option, arg);
-  return non_printable_types[i];
-}
-
-/*
- * Return a major mode
- */  
-static const char *const major_args[] =
-{
-  "rows", "columns", 0
-};
-
-static MAJOR major_types[] =
-{
-  major_rows, major_columns
-};
-
-static MAJOR
-get_major (const char * option, const char * arg)
-{
-  int i = a2ps_get_symbolic_value (major_args, major_types,
-				  option, arg);
-  return major_types[i];
-}
-
-/*
- * Return the ratio to inch
- */  
-static const char *const length_args[] =
-{
-  "inchs",
-  "points",
-  "cm", "centimeters",
-  0
-};
-
-static float length_types[] =
-{
-  1.0,
-  72.0,
-  2.54, 2.54
-};
-
-/*
- * Return a value in inches.
- */
-static float
-get_length (const char * option, const char * arg, float min, float max)
-{
-  float res;
-  int i;
-  char buf[256];
-
-  switch (sscanf (arg, "%f%255s", &res, buf)) {
-  case 2:
-    fprintf (stderr, "%f  %s\n", res, buf);
-    i = a2ps_get_symbolic_value (length_args, length_types,
-				 option, buf);
-    res *= length_types [i];
-    break;
-
-  case 1:
-    break;
-
-  default:
-    error (1, 0, "scanf failed");
-  }
-  
-  if (((min >= 0) && (res < min))
-      || ((max >= 0) && (res > max)))
-    {
-      error (0, 0, _("invalid argument `%s' for option `%s'"),
-	     arg, option);
-      fprintf (stderr,
-	       _("Valid arguments are lengths between %.1fin and %.1fin\n"),
-	       min, max);
-      exit (EXIT_BADARG);
-    }
-  
-  return res;
+  return XARGCASEMATCH (option, arg, non_printable_args, non_printable_types);
 }
 
 /************************************************************************/
@@ -457,85 +293,48 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
       /* The option has not been recognized by the application,
        * let the lib handle it */
       switch (c) {
+#define define_nup(_col_,_row_,_orient_)	\
+	job->columns = _col_;		\
+	job->rows = _row_;		\
+	job->orientation = _orient_;	\
+	job->columns_requested = 80;	\
+	job->lines_requested = 0;	\
+	job->madir = madir_rows;
+
       case '1':				/* 1 logical page per sheet */
-	job->columns = 1;
-	job->rows = 1;
-	job->orientation = portrait;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (1, 1,  portrait);
 	break;
 
       case '2':				/* twin pages */
-	job->columns = 2;
-	job->rows = 1;
-	job->orientation = landscape;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (2, 1,  landscape);
 	break;
 
       case '3':				/* 3 virtual pages */
-	job->columns = 3;
-	job->rows = 1;
-	job->orientation = landscape;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (3, 1,  landscape);
 	break;
 
       case '4':				/* 4 virtual pages */
-	job->columns = 2;
-	job->rows = 2;
-	job->orientation = portrait;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (2, 2,  portrait);
 	break;
 
       case '5':				/* 5 virtual pages */
-	job->columns = 5;
-	job->rows = 1;
-	job->orientation = landscape;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (5, 1,  landscape);
 	break;
 
       case '6':				/* 6 virtual pages */
-	job->columns = 3;
-	job->rows = 2;
-	job->orientation = landscape;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (3, 2,  landscape);
 	break;
 
       case '7':				/* 7 virtual pages */
-	job->columns = 7;
-	job->rows = 1;
-	job->orientation = landscape;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (7, 1,  landscape);
 	break;
 
       case '8':				/* 8 virtual pages */
-	job->columns = 4;
-	job->rows = 2;
-	job->orientation = landscape;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (4, 2,  landscape);
 	break;
 
       case '9':				/* 9 virtual pages */
-	job->columns = 3;
-	job->rows = 3;
-	job->orientation = portrait;
-	job->columns_requested = 80;
-	job->lines_requested = 0;
-	job->Major = major_rows;
+	define_nup (3, 3,  portrait);
 	break;
 
       case '=':				/* A user option */
@@ -598,7 +397,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 
       case 152:				/* line numbering */
 	job->numbering = 
-	  a2ps_get_integer_in_range ("--line-numbers", optarg, 0, -2);
+	  get_integer_in_range ("--line-numbers", optarg, 0, 0, range_min);
 	break;
 
       case 'd':				/* fork a process to print */ 
@@ -629,7 +428,8 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	  else
 	    cp = optarg;
 	  /* A font size is given */
-	  job->fontsize = get_length ("--font-size", cp, 2.0, 200.0);
+	  job->fontsize = get_length ("--font-size", cp, 
+				      0.0, 0.0, "pt", range_min_strict);
 	  job->columns_requested = 0;
 	  job->lines_requested = 0;	  
 	}
@@ -662,7 +462,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
       case 'l':
 	/* set columns per line, useful for most cases */
 	job->columns_requested = 
-	  a2ps_get_integer_in_range ("--chars-per-line", optarg, 1, -2);
+	  get_integer_in_range ("--chars-per-line", optarg, 1, 0, range_min);
 	job->lines_requested = 0;
 	break;
 
@@ -670,7 +470,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	/* set lines per page.  Useful with preformatted files. Scaling is
 	 * automatically done when necessary.  */
 	job->lines_requested = 
-	  a2ps_get_integer_in_range ("--lines-per-page", optarg, 1, -2);
+	  get_integer_in_range ("--lines-per-page", optarg, 1, 0, range_min);
 	/* Unset value given to columns-per-page, so that this one
 	 * is not hidden */
 	job->columns_requested = 0;
@@ -687,7 +487,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 
       case 'n':				/* n copies */
 	job->copies = 
-	  a2ps_get_integer_in_range ("--copies", optarg, 0, -2);
+	  get_integer_in_range ("--copies", optarg, 1, 0, range_min);
 	break;
 
       case 'o':			/* output goes into a file */
@@ -711,7 +511,8 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 's':
-	switch (a2ps_get_integer_in_range ("--sides", optarg, 1, 2)) {
+	switch (get_integer_in_range ("--sides", optarg,
+				      1, 2, range_min_max)) {
 	case 1:
 	  job->duplex = 0;
 	  delpagedevice (job, "Duplex");
@@ -747,7 +548,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 
       case 'T':
 	job->tabsize = 
-	  a2ps_get_integer_in_range ("--tabsize", optarg, 1, -2);
+	  get_integer_in_range ("--tabsize", optarg, 1, 0, range_min);
 	break;
 
       case 'u':				/* water mark (under lay) */
@@ -757,7 +558,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
       case 'v':					    /* verbosity */
 	if (!IS_EMPTY(optarg)) {
 	  message_verbosity = 
-	    a2ps_get_integer_in_range ("--verbose", optarg, 0, -2);
+	    get_integer_in_range ("--verbose", optarg, 0, 0, range_min);
 	} else
 	  message_verbosity = msg_report1;
 	break;
@@ -778,12 +579,12 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 
       case 132:				/* Number of columns */
 	job->columns = 
-	  a2ps_get_integer_in_range ("--columns", optarg, 1, -2);
+	  get_integer_in_range ("--columns", optarg, 1, 0, range_min);
 	break;
 
       case 133:				/* Number of rows */
 	job->rows =
-	  a2ps_get_integer_in_range ("--rows", optarg, 1, -2);
+	  get_integer_in_range ("--rows", optarg, 1, 0, range_min);
 	break;
 
       case 134:				/* --include ps prologue */
@@ -846,7 +647,7 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
       case 147:				/* --margin */
 	if (optarg)
 	  job->margin =
-	    a2ps_get_integer_in_range ("--margin", optarg, 0, -2);
+	    get_integer_in_range ("--margin", optarg, 0, 0, range_min);
 	else
 	  job->margin = 12;
 	break;
@@ -856,11 +657,11 @@ a2ps_handle_option (a2ps_job * job, int argc, char *argv[])
 	break;
 
       case 157: 			/* --major= */
-	job->Major = get_major ("--major", optarg);
+	job->madir = madir_argmatch ("--major", optarg);
 	break;
 
       case 158:				/* --version-control */
-	backup_type = a2ps_get_backup_type (optarg);
+	job->backup_type = a2ps_get_backup_type (optarg);
 	break;
 
       case 159:				/* --suffix	*/
