@@ -31,11 +31,6 @@
 #include "routines.h"
 #include "filtdir.h"
 
-/*
- * From savedir.c (1st arg is _not_ const in savedir)
- */
-char * savedir PARAMS ((const char *dir, unsigned name_size));
-
 /*
  * Build in a alloca'ed zone the concat of DIR and FILE
  */
