@@ -424,8 +424,8 @@ output_document_setup (a2ps_job * job)
     output (jdiv, "/pmh 0 def\n");
   output (jdiv, "/v 0 def\n");
   
-  switch (job->Major) {
-  case major_rows:
+  switch (job->madir) {
+  case madir_rows:
     output (jdiv, "/x [\n");
     for (j = 1 ; j <= job->rows ; j++) {
       output (jdiv, "  0\n");
@@ -443,7 +443,7 @@ output_document_setup (a2ps_job * job)
     output (jdiv, "] def\n");
     break;
 
-  case major_columns:
+  case madir_columns:
     output (jdiv, "/x [\n");
     for (i = 1 ; i <= job->columns ; i++) {
       output (jdiv, "  pmw pw add %d mul\n", i - 1);
