@@ -78,7 +78,7 @@
  * Returns number of columns used (on the output) to print the char.
  */
 static int 
-format_char (a2ps_job * job, uchar c, uchar * res)
+ps_escape_char (a2ps_job * job, uchar c, uchar * res)
 {
   int len=0;
 
@@ -103,8 +103,8 @@ format_char (a2ps_job * job, uchar c, uchar * res)
   /* We are dealing with a non-printing character */
   job->status->nonprinting_chars++;
 
-  /* We do this in here for speed.  We could have first escaped,
-  * then consider the new string as the string to ps-escape. */
+  /* We do this in here for speed.  We could have first escaped, then
+    consider the new string as the string to ps-escape. */
   switch (job->unprintable_format) {
   case octal:
     sprintf ((char *) res, "\\\\%03o", c);
@@ -115,11 +115,11 @@ format_char (a2ps_job * job, uchar c, uchar * res)
     return 4;
 
   case question_mark:
-    USTRCCAT(res, '?');
+    USTRCCAT (res, '?');
     return 1;
 
   case space:
-    USTRCCAT(res, ' ');
+    USTRCCAT (res, ' ');
     return 1;
     
   case caret:
@@ -132,7 +132,8 @@ format_char (a2ps_job * job, uchar c, uchar * res)
     if (c < ' ') {
       USTRCCAT(res, '^');
       len += 2; 
-      if ((c = c + '@') == '(' || c == ')' || c == '\\')
+      c += '@';
+      if (c == '(' || c == ')' || c == '\\')
 	USTRCCAT(res, '\\');
       USTRCCAT(res, c);
     } else if (c == 0177) {
@@ -156,7 +157,8 @@ format_char (a2ps_job * job, uchar c, uchar * res)
     if (c < ' ') {
       ustrcat (res, "C-");
       len += 3; 
-      if ((c = c + '`') == '(' || c == ')' || c == '\\')
+      c += '@';
+      if (c == '(' || c == ')' || c == '\\')
 	USTRCCAT(res, '\\');
       USTRCCAT(res, c);
     } else if (c == 0177) {
@@ -171,20 +173,20 @@ format_char (a2ps_job * job, uchar c, uchar * res)
     return len;
 
   }
-  return (0);
+  return 0;
 }
 
 /*
  * Print a string in a form accepted by postscript printers.
  */
 static int 
-format_string (a2ps_job * job, uchar * string, uchar * res)
+ps_escape_string (a2ps_job * job, uchar * string, uchar * res)
 {
   size_t i;
   int delta_column=0;
 
   for (i = 0 ; i < ustrlen (string) ; i++) 
-    delta_column += format_char (job, string[i], res);
+    delta_column += ps_escape_char (job, string[i], res);
   
   return delta_column;
 }
@@ -229,7 +231,7 @@ output_marker (a2ps_job * job, const char * kind, uchar * marker)
       break;
     default:
       *buf = NUL;
-      format_char (job, cp[i], buf);
+      ps_escape_char (job, cp[i], buf);
       output (jdiv, (char *) buf);
       break;
     }
@@ -856,16 +858,14 @@ ps_print_char (a2ps_job * job, int c, enum face_e new_face)
     job->status->face_declared = FALSE;
   }
   
-      nchars = format_char (job, c, buf);
-      job->status->wx += 
-	((nchars == 1) ? char_WX (job, c) : string_WX (job, buf));
+      nchars = ps_escape_char (job, c, buf);
+      job->status->wx += char_WX (job, c);
       job->status->column += nchars;
       if (line_full) {
 	if (job->folding) {
 	  fold_line (job, new_face);
 	  job->status->column = nchars;
-	  job->status->wx = 
-	    ((nchars == 1) ? char_WX (job, c) : string_WX (job, buf));
+	  job->status->wx = char_WX (job, c);
 	} else {
 	  job->status->is_in_cut = TRUE;
 	  return;
