@@ -3,8 +3,8 @@
  *
  * routines for the postscript output
  * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
- * Copyright (c) 1995, 96, 97 Akim Demaille, Miguel Santana
- * $Id: psgen.h,v 1.1.2.1 1997/05/26 08:25:22 demaille Exp $
+ * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
+ *
  */
 
 /*
@@ -25,15 +25,40 @@
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+
+/*
+ * $Id: psgen.h,v 1.16 1998/03/02 08:57:12 demaille Exp $
+ */
+
 #ifndef _PSGEN_H_
 #define _PSGEN_H_
 
-#include "liba2ps.h"
+struct a2ps_job;
+struct encoding;
 
-void ps_init PARAMS ((struct print_job * job));
-void ps_close PARAMS ((struct print_job * job));
-void ps_print_char PARAMS ((struct print_job * job, int c, FACE true_font));
-void ps_print_string PARAMS ((struct print_job * job, ustring string, FACE new_font));
-void ps_end_file PARAMS ((struct print_job * job));
+#include "faces.h"
+
+void ps_print_char PARAMS ((struct a2ps_job * job, 
+			    int c, enum face_e face));
+void ps_print_string PARAMS ((struct a2ps_job * job, 
+			      uchar * string, enum face_e face));
+void ps_print_buffer PARAMS ((struct a2ps_job * job,
+			      const uchar * buffer,
+			      size_t start, size_t end,
+			      enum face_e face));
+void ps_end_file PARAMS ((struct a2ps_job * job));
+
+void clean_up PARAMS ((struct a2ps_job * job));
+void require_fresh_page PARAMS ((struct a2ps_job * job));
+
+/*
+ * Multi-lingual routines
+ */
+
+void ps_set_encoding PARAMS ((struct a2ps_job * job,
+			      struct encoding * encoding));
+void ps_end_encoding PARAMS ((struct a2ps_job * job));
+void ps_switch_encoding PARAMS ((struct a2ps_job * job,
+				 struct encoding * encoding));
 
 #endif
