@@ -30,57 +30,7 @@
 #include "routines.h"
 #include "stpncpy.h"
 #include "msg.h"
-#include "backupfile.h"
 
-/*
- * If the file exists, and the backup machinery is on,
- * make the backup (i.e. rename the file etc.)
- */
-void
-xbackup_file (const char * filename)
-{
-  char * backup_name;
-  struct stat filestat;
-
-  /* If not backup required, or file is /dev/null, do nothing */
-  if ((backup_type == none) || strequ (filename, "/dev/null"))
-    return ;
-
-  if (stat (filename, &filestat))
-    {
-      if (errno == ENOENT)
-	/* the file does not exist: return */
-	return ;
-      else
-	/* Another kind of error occured: exit */
-	error (1, errno, _("cannot get informations on file `%s'"),
-	       filename);
-    }
-
-  /* The file exists: make a backup */
-  backup_name = xfind_backup_file_name (filename);
-  if (rename (filename, backup_name))
-    error (1, errno, _("cannot rename file `%s' as `%s'"),
-		       filename, backup_name);
-  FREE (backup_name);
-}
-
-
-/*
- * Easy call to find_backup_file_name
- */
-char *
-xfind_backup_file_name (const char * file)
-{
-  char * res;
-
-  res = find_backup_file_name (file);
-  if (!res)
-    error (1, 0, _("Memory exhausted"));
-
-  return res;
-}
-
 /*
  * Convert a list of string of valid chars to an yes/no array
  */
