@@ -187,10 +187,6 @@ char * stpncpy PARAMS ((char * dest, const char * src, size_t n));
 #define ustrequ(s1, s2) (!ustrcmp(s1, s2))
 #define strcaseequ(s1, s2) (!strcasecmp(s1, s2))
 
-/* Wrapping of find_backup_file_name */
-void xbackup_file PARAMS ((const char * filename));
-char * xfind_backup_file_name PARAMS ((const char * file));
-
 /* Functions on strings */
 void string_to_array PARAMS ((uchar arr[256], const uchar * string));
 int is_strlower PARAMS((const uchar * string));
