@@ -18,6 +18,14 @@
 #ifndef _xstring_h_
 #define _xstring_h_
 
+#ifndef PARAMS
+# if defined (__GNUC__) || __STDC__
+#  define PARAMS(args) args
+# else
+#  define PARAMS(args) ()
+# endif
+#endif
+
 #define dirname sidestep_dirname_declaration
 #define basename sidestep_basename_declaration
 #ifdef HAVE_STRING_H
@@ -32,14 +40,24 @@ char *memchr ();
 #undef dirname
 #undef basename
 
+#if !defined(HAVE_STRCHR) && !defined(strchr)
+# define strchr index
+# define strrchr rindex
+#endif
+extern char *strchr  PARAMS ((const char *s, int c));
+extern char *strrchr PARAMS ((const char *s, int c));
+
+#  ifndef HAVE_STRERROR
+extern char *strerror PARAMS ((int));
+#  endif
+
 #define strequ(s1, s2)		(strcmp ((s1), (s2)) == 0)
 #define strnequ(s1, s2, n)	(strncmp ((s1), (s2), (n)) == 0)
 
-#if !HAVE_STRNDUP
-extern char *strndup __P((char const *, size_t n));
+#ifndef HAVE_STRNDUP
+extern char *strndup PARAMS ((char const *, size_t n));
 #endif
-/* This breaks on OSF
-extern char *basename __P((char const *));
-extern char *dirname __P((char const *));
-*/
+extern char *basename PARAMS ((char const *));
+extern char *dirname PARAMS ((char const *));
+
 #endif /* not _xstring_h_ */
