@@ -1,6 +1,6 @@
 /* xstrrpl.h -- declaration of function for replacement of substrings
    Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
-   Copyright (c) 1995, 96, 97 Akim Demaille, Miguel Santana
+   Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,9 +22,9 @@
 #define _XSTRRPL_H_
 
 # if defined (__STDC__) && __STDC__
-extern char * xstrrpl (char * string, char * subst[][2]);
-extern void strrpl (char ** string, char * subst[][2]);
-extern char * xvstrrpl (char * string, ...);
+extern char * xstrrpl (const char * string, const char * subst[][2]);
+extern void strrpl (char ** string, const char * subst[][2]);
+extern char * xvstrrpl (const char * string, ...);
 extern void vstrrpl (char ** string, ...);
 # else
 char * xstrrpl ();
