@@ -4,7 +4,7 @@
  * routines of input
  * Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana
  * Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana
- * $Id: buffer.c,v 1.74 1998/04/04 12:43:15 demaille Exp $
+ * $Id: buffer.c,v 1.77 1998/08/27 09:44:44 demaille Exp $
  */
 
 /*
@@ -27,14 +27,15 @@
  */
 
 /*
- * $Id: buffer.c,v 1.74 1998/04/04 12:43:15 demaille Exp $
+ * $Id: buffer.c,v 1.77 1998/08/27 09:44:44 demaille Exp $
  */
 
 #include "a2ps.h"
 #include "buffer.h"
 #include "routines.h"
 #include "xobstack.h"
-#include "options.h"
+#include "argmatch.h"
+
 
 /****************************************************************/
 /*		 	Handling of the various eol styles	*/
@@ -93,9 +94,7 @@ enum eol_e
 option_string_to_eol (const char * option,
 		      const char * arg)
 {
-  int i = a2ps_get_symbolic_value (eol_args, eol_types,
-				   option, arg);
-  return eol_types[i];
+  return XARGCASEMATCH (option, arg, eol_args, eol_types);
 }
 
 /****************************************************************/
@@ -434,7 +433,7 @@ buffer_sample_get (buffer_t * buffer, const char * filename)
   FILE * out = xwfopen (filename);
   size_t cur = 0;
   int c;
-  uchar * sample_buffer = ALLOC (uchar, SAMPLE_SIZE);
+  char * sample_buffer = MALLOC (SAMPLE_SIZE);
 
   for (; (cur < SAMPLE_SIZE) && ((c = getc (buffer->stream)) != EOF) ; cur++)
     {
