@@ -56,6 +56,8 @@
 #include "psgen.h"
 #include "prolog.h"
 #include "stream.h"
+#include "argmatch.h"
+#include "getnum.h"
 
 /************************************************************************/
 /*									*/
@@ -185,10 +187,8 @@ static const int highlight_level_types[] =
 static int
 get_highlight_level (const char * option, const char * arg)
 {
-  int i = a2ps_get_symbolic_value (highlight_level_args,
-				   highlight_level_types,
-				   option, arg);
-  return highlight_level_types[i];
+  return XARGCASEMATCH (option, arg, 
+                        highlight_level_args, highlight_level_types);
 }
 
 static char *
@@ -251,23 +251,21 @@ static int
 get_verbosity (const char * option, char * arg)
 {
 #define verbosity_sep ",:;+"
-  int res = 0, i;
+  int res = 0;
   char * token;
 
   if (isdigit ((int) *arg))
     {
       /* The verbosity is set through an integer value */
       message_verbosity = 
-	a2ps_get_integer_in_range ("--verbose", arg, 0, -2);
+	get_integer_in_range ("--verbose", arg, 0, 0, range_min);
     } 
   else
     {
       /* Keywords are used to define verbosity level */
       token = strtok (arg, verbosity_sep);
       do {
-	i = a2ps_get_symbolic_value (verbosity_args, verbosity_types,
-				     option, token);
-	res |= verbosity_types[i];
+	res |= XARGCASEMATCH (option, token, verbosity_args, verbosity_types);
       } while ((token = strtok (NULL, verbosity_sep)));
     }
   
@@ -331,9 +329,7 @@ static const enum behavior behavior_types[] =
 static enum behavior
 get_behavior (const char * option, const char * arg)
 {
-  int i = a2ps_get_symbolic_value (behavior_args, behavior_types,
-				   option, arg);
-  return behavior_types[i];
+  return XARGCASEMATCH (option, arg, behavior_args, behavior_types);
 }
 
 /************************************************************************/
@@ -366,7 +362,17 @@ static void
 version (FILE * stream)
 {
   fprintf (stream, 
-	   "%s %s\n", PACKAGE, VERSION);
+	   "GNU %s %s\n", PACKAGE, VERSION);
+
+  fputs (_("\
+\n\
+Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana\n\
+Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana\n"),
+             stream);
+  fputs (_("\
+This is free software; see the source for copying conditions.  There is NO\n\
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
+	 stream);
 
   /* TRANS: Don't forget to add something about your own translation, e.g.
      Written by <Akim.Demaille@inf.enst.fr> and <Miguel.Santana@st.com>
@@ -377,15 +383,6 @@ version (FILE * stream)
 Written by <Akim.Demaille@inf.enst.fr> and <Miguel.Santana@st.com>\n\
 News, updates and documentation: visit http://www.inf.enst.fr/~demaille/a2ps/\n"), 
 	 stream);
-  fputs (_("\
-\n\
-Copyright (c) 1988, 89, 90, 91, 92, 93 Miguel Santana\n\
-Copyright (c) 1995, 96, 97, 98 Akim Demaille, Miguel Santana\n"),
-             stream);
-  fputs (_("\
-This is free software; see the source for copying conditions.  There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"),
-	 stdout);
 }
 
 /*
@@ -413,7 +410,7 @@ list_options (struct a2ps_job * a_job, FILE * stream)
 	   a_job->medium->name,
 	   a_job->columns, a_job->rows, 
 	   (a_job->orientation == portrait) ? _("portrait") : _("landscape"),
-	   a_job->Major == major_rows ? _("rows first") : _("columns first"),
+	   madir_to_string (a_job->madir),
 	   a_job->border ? _("yes") : _("no"),
 	   a_job->compact_mode ? _("yes") : _("no"),
 	   a_job->margin);
@@ -425,7 +422,7 @@ list_options (struct a2ps_job * a_job, FILE * stream)
     sprintf ((char *) buf, _("%d lines per page"), 
 	     a_job->lines_requested);
   else
-    sprintf ((char *) buf, _("font size is %.0fin"), a_job->fontsize);
+    sprintf ((char *) buf, _("font size is %gpt"), a_job->fontsize);
 
   switch (a_job->numbering) {
   case 0:
@@ -515,7 +512,7 @@ list_options (struct a2ps_job * a_job, FILE * stream)
   destination_to_string (a_job, buf);
   
   /* Make a nice message to tell what version control is used */
-  switch (backup_type) {
+  switch (a_job->backup_type) {
   case none:
     cp = _("never make backups");
     break;
@@ -719,7 +716,7 @@ TOPIC can be any non-ambiguous abbreviation of:\n\
   -v, --verbose[=LEVEL]      set verbosity on, or to LEVEL\n\
   -=, --user-option=OPTION   use the user defined shortcut OPTION\n\
   --debug                    enable debugging features\n\
-  --variable=KEY[:VALUE]     unset or set the macro K to the value V\n"),
+  --define=KEY[:VALUE]       unset variable KEY or set to VALUE\n"),
 	 stream);
 
   putc ('\n', stream);
@@ -785,7 +782,7 @@ TOPIC can be any non-ambiguous abbreviation of:\n\
   -E, --pretty-print[=LANG]  enable pretty-printing (set style to LANG)\n\
   --highlight-level=LEVEL    set pretty printing highlight LEVEL\n\
                              LEVEL can be none, normal or heavy\n\
-  -g                         alias for --highligh-level=heavy\n\
+  -g                         alias for --highlight-level=heavy\n\
   --strip-level=NUM          level of comments stripping\n"),
 	 stream);
 
@@ -815,9 +812,9 @@ TOPIC can be any non-ambiguous abbreviation of:\n\
   putc ('\n', stream);
   /* TRANS: This is the second place where you are invited to drop
    * a mail to report translation problems. Eg.:
-   Report bugs to <a2ps-bugs@inf.enst.fr>
+   Report bugs to <a2ps-bugs@gnu.org>
    Report translation problems to <me@there> */
-  fputs (_("Report bugs to <a2ps-bugs@inf.enst.fr>\n"),
+  fputs (_("Report bugs to <bug-a2ps@gnu.org>\n"),
 	 stream);
 }
 
@@ -881,7 +878,8 @@ handle_a2ps_option (int option, char * optional_arg)
     
   case 148:				/* --strip-level */
     strip_level =
-      a2ps_get_integer_in_range ("--strip-level", optional_arg, 0, 3);
+      get_integer_in_range ("--strip-level", optional_arg, 
+			    0, 3, range_min_max);
     break;
     
   case 160:				/* --delegate=BOOL */
@@ -962,12 +960,14 @@ main (int argc, char *argv[])
 
   /* Personal config.: only when installed, because there
    * may be incompatibilities between config files versions */
-  if (! (getenv ("NO_SYSCONF")))
-    (void) a2_read_config (job, job->user->home, ".a2ps/a2psrc");
-  
-  /* Local config. */
-  (void) a2_read_config (job, ".", ".a2psrc");
-
+  if (! (getenv ("NO_HOMECONF")))
+    {
+      /* Home's */
+      (void) a2_read_config (job, job->user->home, ".a2ps/a2psrc");
+      
+      /* Local config. */
+      (void) a2_read_config (job, ".", ".a2psrc");
+    }
   /* Prepare the sheets map structure and the hash table that
    * receives yet read sheets */
   sheets_map = sheets_map_new ();
